@@ -40,12 +40,13 @@ ser_permutation_vector <- function(x, method = NULL) {
 
 ## accessors
 get_order <- function(x, ...) UseMethod("get_order")
+
 get_order.ser_permutation_vector <- function(x, ...) NextMethod()
-get_order.hclust <- function(x, ...) {
-  o <- x$order
-  #names(o) <- x$labels[o]
-  o
-}
+
+get_order.hclust <- function(x, ...) x$order
+
+get_order.dendrogram <- function(x, ...) order.dendrogram(x)
+
 
 get_order.integer <- function(x, ...) {
   o <- as.integer(x)
